@@ -240,3 +240,23 @@ pub mod rfc_4638_executor {}
 /// Remove metakey
 #[doc = include_str!("5314_remove_metakey.md")]
 pub mod rfc_5314_remove_metakey {}
+
+/// Operator from uri
+#[doc = include_str!("5444_operator_from_uri.md")]
+pub mod rfc_5444_operator_from_uri {}
+
+/// Context
+#[doc = include_str!("5479_context.md")]
+pub mod rfc_5479_context {}
+
+/// Conditional Reader
+#[doc = include_str!("5485_conditional_reader.md")]
+pub mod rfc_5485_conditional_reader {}
+
+/// List With Deleted
+#[doc = include_str!("5495_list_with_deleted.md")]
+pub mod rfc_5495_list_with_deleted {}
+
+/// Write Returns Metadata
+#[doc = include_str!("5556_write_returns_metadata.md")]
+pub mod rfc_5556_write_returns_metadata {}
