@@ -17,17 +17,8 @@
 
 use std::fmt::Debug;
 use std::fmt::Formatter;
-use std::io::SeekFrom;
-use std::task::Context;
-use std::task::Poll;
+use std::sync::Arc;
 
-use async_trait::async_trait;
-use bytes::Bytes;
-use futures::TryFutureExt;
-
-use crate::raw::oio::ListOperation;
-use crate::raw::oio::ReadOperation;
-use crate::raw::oio::WriteOperation;
 use crate::raw::*;
 use crate::*;
 
@@ -35,37 +26,44 @@ use crate::*;
 ///
 /// # Notes
 ///
-/// This layer will adding the following error context into all errors:
+/// This layer will add the following error context into all errors:
 ///
 /// - `service`: The [`Scheme`] of underlying service.
 /// - `operation`: The [`Operation`] of this operation
 /// - `path`: The path of this operation
+///
+/// Some operations may have additional context:
+///
+/// - `range`: The range the read operation is trying to read.
+/// - `read`: The already read size in given reader.
+/// - `size`: The size of the current write operation.
+/// - `written`: The already written size in given writer.
+/// - `listed`: The already listed size in given lister.
+/// - `deleted`: The already deleted size in given deleter.
 pub struct ErrorContextLayer;
 
-impl<A: Accessor> Layer<A> for ErrorContextLayer {
-    type LayeredAccessor = ErrorContextAccessor<A>;
+impl<A: Access> Layer<A> for ErrorContextLayer {
+    type LayeredAccess = ErrorContextAccessor<A>;
 
-    fn layer(&self, inner: A) -> Self::LayeredAccessor {
-        let meta = inner.info();
-        ErrorContextAccessor { meta, inner }
+    fn layer(&self, inner: A) -> Self::LayeredAccess {
+        let info = inner.info();
+        ErrorContextAccessor { info, inner }
     }
 }
 
 /// Provide error context wrapper for backend.
-pub struct ErrorContextAccessor<A: Accessor> {
-    meta: AccessorInfo,
+pub struct ErrorContextAccessor<A: Access> {
+    info: Arc<AccessorInfo>,
     inner: A,
 }
 
-impl<A: Accessor> Debug for ErrorContextAccessor<A> {
+impl<A: Access> Debug for ErrorContextAccessor<A> {
     fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
         self.inner.fmt(f)
     }
 }
 
-#[cfg_attr(not(target_arch = "wasm32"), async_trait)]
-#[cfg_attr(target_arch = "wasm32", async_trait(?Send))]
-impl<A: Accessor> LayeredAccessor for ErrorContextAccessor<A> {
+impl<A: Access> LayeredAccess for ErrorContextAccessor<A> {
     type Inner = A;
     type Reader = ErrorContextWrapper<A::Reader>;
     type BlockingReader = ErrorContextWrapper<A::BlockingReader>;
@@ -73,168 +71,125 @@ impl<A: Accessor> LayeredAccessor for ErrorContextAccessor<A> {
     type BlockingWriter = ErrorContextWrapper<A::BlockingWriter>;
     type Lister = ErrorContextWrapper<A::Lister>;
     type BlockingLister = ErrorContextWrapper<A::BlockingLister>;
+    type Deleter = ErrorContextWrapper<A::Deleter>;
+    type BlockingDeleter = ErrorContextWrapper<A::BlockingDeleter>;
 
     fn inner(&self) -> &Self::Inner {
         &self.inner
     }
 
-    fn metadata(&self) -> AccessorInfo {
-        self.meta.clone()
+    fn info(&self) -> Arc<AccessorInfo> {
+        self.info.clone()
     }
 
     async fn create_dir(&self, path: &str, args: OpCreateDir) -> Result<RpCreateDir> {
-        self.inner
-            .create_dir(path, args)
-            .map_err(|err| {
-                err.with_operation(Operation::CreateDir)
-                    .with_context("service", self.meta.scheme())
-                    .with_context("path", path)
-            })
-            .await
+        self.inner.create_dir(path, args).await.map_err(|err| {
+            err.with_operation(Operation::CreateDir)
+                .with_context("service", self.info.scheme())
+                .with_context("path", path)
+        })
     }
 
     async fn read(&self, path: &str, args: OpRead) -> Result<(RpRead, Self::Reader)> {
-        let br = args.range();
-
+        let range = args.range();
         self.inner
             .read(path, args)
-            .map_ok(|(rp, r)| {
+            .await
+            .map(|(rp, r)| {
                 (
                     rp,
-                    ErrorContextWrapper {
-                        scheme: self.meta.scheme(),
-                        path: path.to_string(),
-                        inner: r,
-                    },
+                    ErrorContextWrapper::new(self.info.scheme(), path.to_string(), r)
+                        .with_range(range),
                 )
             })
             .map_err(|err| {
                 err.with_operation(Operation::Read)
-                    .with_context("service", self.meta.scheme())
+                    .with_context("service", self.info.scheme())
                     .with_context("path", path)
-                    .with_context("range", br.to_string())
+                    .with_context("range", range.to_string())
             })
-            .await
     }
 
     async fn write(&self, path: &str, args: OpWrite) -> Result<(RpWrite, Self::Writer)> {
         self.inner
             .write(path, args)
-            .map_ok(|(rp, w)| {
+            .await
+            .map(|(rp, w)| {
                 (
                     rp,
-                    ErrorContextWrapper {
-                        scheme: self.meta.scheme(),
-                        path: path.to_string(),
-                        inner: w,
-                    },
+                    ErrorContextWrapper::new(self.info.scheme(), path.to_string(), w),
                 )
             })
             .map_err(|err| {
                 err.with_operation(Operation::Write)
-                    .with_context("service", self.meta.scheme())
+                    .with_context("service", self.info.scheme())
                     .with_context("path", path)
             })
-            .await
     }
 
     async fn copy(&self, from: &str, to: &str, args: OpCopy) -> Result<RpCopy> {
-        self.inner
-            .copy(from, to, args)
-            .map_err(|err| {
-                err.with_operation(Operation::Copy)
-                    .with_context("service", self.meta.scheme())
-                    .with_context("from", from)
-                    .with_context("to", to)
-            })
-            .await
+        self.inner.copy(from, to, args).await.map_err(|err| {
+            err.with_operation(Operation::Copy)
+                .with_context("service", self.info.scheme())
+                .with_context("from", from)
+                .with_context("to", to)
+        })
     }
 
     async fn rename(&self, from: &str, to: &str, args: OpRename) -> Result<RpRename> {
-        self.inner
-            .rename(from, to, args)
-            .map_err(|err| {
-                err.with_operation(Operation::Rename)
-                    .with_context("service", self.meta.scheme())
-                    .with_context("from", from)
-                    .with_context("to", to)
-            })
-            .await
+        self.inner.rename(from, to, args).await.map_err(|err| {
+            err.with_operation(Operation::Rename)
+                .with_context("service", self.info.scheme())
+                .with_context("from", from)
+                .with_context("to", to)
+        })
     }
 
     async fn stat(&self, path: &str, args: OpStat) -> Result<RpStat> {
-        self.inner
-            .stat(path, args)
-            .map_err(|err| {
-                err.with_operation(Operation::Stat)
-                    .with_context("service", self.meta.scheme())
-                    .with_context("path", path)
-            })
-            .await
+        self.inner.stat(path, args).await.map_err(|err| {
+            err.with_operation(Operation::Stat)
+                .with_context("service", self.info.scheme())
+                .with_context("path", path)
+        })
     }
 
-    async fn delete(&self, path: &str, args: OpDelete) -> Result<RpDelete> {
+    async fn delete(&self) -> Result<(RpDelete, Self::Deleter)> {
         self.inner
-            .delete(path, args)
+            .delete()
+            .await
+            .map(|(rp, w)| {
+                (
+                    rp,
+                    ErrorContextWrapper::new(self.info.scheme(), "".to_string(), w),
+                )
+            })
             .map_err(|err| {
                 err.with_operation(Operation::Delete)
-                    .with_context("service", self.meta.scheme())
-                    .with_context("path", path)
+                    .with_context("service", self.info.scheme())
             })
-            .await
     }
 
     async fn list(&self, path: &str, args: OpList) -> Result<(RpList, Self::Lister)> {
         self.inner
             .list(path, args)
-            .map_ok(|(rp, p)| {
+            .await
+            .map(|(rp, p)| {
                 (
                     rp,
-                    ErrorContextWrapper {
-                        scheme: self.meta.scheme(),
-                        path: path.to_string(),
-                        inner: p,
-                    },
+                    ErrorContextWrapper::new(self.info.scheme(), path.to_string(), p),
                 )
             })
             .map_err(|err| {
                 err.with_operation(Operation::List)
-                    .with_context("service", self.meta.scheme())
+                    .with_context("service", self.info.scheme())
                     .with_context("path", path)
             })
-            .await
-    }
-
-    async fn batch(&self, args: OpBatch) -> Result<RpBatch> {
-        self.inner
-            .batch(args)
-            .map_ok(|v| {
-                let res = v
-                    .into_results()
-                    .into_iter()
-                    .map(|(path, res)| {
-                        let res = res.map_err(|err| {
-                            err.with_operation(Operation::Delete)
-                                .with_context("service", self.meta.scheme())
-                                .with_context("path", &path)
-                        });
-                        (path, res)
-                    })
-                    .collect();
-
-                RpBatch::new(res)
-            })
-            .map_err(|err| {
-                err.with_operation(Operation::Batch)
-                    .with_context("service", self.meta.scheme())
-            })
-            .await
     }
 
     async fn presign(&self, path: &str, args: OpPresign) -> Result<RpPresign> {
         self.inner.presign(path, args).await.map_err(|err| {
             err.with_operation(Operation::Presign)
-                .with_context("service", self.meta.scheme())
+                .with_context("service", self.info.scheme())
                 .with_context("path", path)
         })
     }
@@ -242,28 +197,27 @@ impl<A: Accessor> LayeredAccessor for ErrorContextAccessor<A> {
     fn blocking_create_dir(&self, path: &str, args: OpCreateDir) -> Result<RpCreateDir> {
         self.inner.blocking_create_dir(path, args).map_err(|err| {
             err.with_operation(Operation::BlockingCreateDir)
-                .with_context("service", self.meta.scheme())
+                .with_context("service", self.info.scheme())
                 .with_context("path", path)
         })
     }
 
     fn blocking_read(&self, path: &str, args: OpRead) -> Result<(RpRead, Self::BlockingReader)> {
+        let range = args.range();
         self.inner
             .blocking_read(path, args)
             .map(|(rp, os)| {
                 (
                     rp,
-                    ErrorContextWrapper {
-                        scheme: self.meta.scheme(),
-                        path: path.to_string(),
-                        inner: os,
-                    },
+                    ErrorContextWrapper::new(self.info.scheme(), path.to_string(), os)
+                        .with_range(range),
                 )
             })
             .map_err(|err| {
                 err.with_operation(Operation::BlockingRead)
-                    .with_context("service", self.meta.scheme())
+                    .with_context("service", self.info.scheme())
                     .with_context("path", path)
+                    .with_context("range", range.to_string())
             })
     }
 
@@ -273,16 +227,12 @@ impl<A: Accessor> LayeredAccessor for ErrorContextAccessor<A> {
             .map(|(rp, os)| {
                 (
                     rp,
-                    ErrorContextWrapper {
-                        scheme: self.meta.scheme(),
-                        path: path.to_string(),
-                        inner: os,
-                    },
+                    ErrorContextWrapper::new(self.info.scheme(), path.to_string(), os),
                 )
             })
             .map_err(|err| {
                 err.with_operation(Operation::BlockingWrite)
-                    .with_context("service", self.meta.scheme())
+                    .with_context("service", self.info.scheme())
                     .with_context("path", path)
             })
     }
@@ -290,7 +240,7 @@ impl<A: Accessor> LayeredAccessor for ErrorContextAccessor<A> {
     fn blocking_copy(&self, from: &str, to: &str, args: OpCopy) -> Result<RpCopy> {
         self.inner.blocking_copy(from, to, args).map_err(|err| {
             err.with_operation(Operation::BlockingCopy)
-                .with_context("service", self.meta.scheme())
+                .with_context("service", self.info.scheme())
                 .with_context("from", from)
                 .with_context("to", to)
         })
@@ -299,7 +249,7 @@ impl<A: Accessor> LayeredAccessor for ErrorContextAccessor<A> {
     fn blocking_rename(&self, from: &str, to: &str, args: OpRename) -> Result<RpRename> {
         self.inner.blocking_rename(from, to, args).map_err(|err| {
             err.with_operation(Operation::BlockingRename)
-                .with_context("service", self.meta.scheme())
+                .with_context("service", self.info.scheme())
                 .with_context("from", from)
                 .with_context("to", to)
         })
@@ -308,17 +258,24 @@ impl<A: Accessor> LayeredAccessor for ErrorContextAccessor<A> {
     fn blocking_stat(&self, path: &str, args: OpStat) -> Result<RpStat> {
         self.inner.blocking_stat(path, args).map_err(|err| {
             err.with_operation(Operation::BlockingStat)
-                .with_context("service", self.meta.scheme())
+                .with_context("service", self.info.scheme())
                 .with_context("path", path)
         })
     }
 
-    fn blocking_delete(&self, path: &str, args: OpDelete) -> Result<RpDelete> {
-        self.inner.blocking_delete(path, args).map_err(|err| {
-            err.with_operation(Operation::BlockingDelete)
-                .with_context("service", self.meta.scheme())
-                .with_context("path", path)
-        })
+    fn blocking_delete(&self) -> Result<(RpDelete, Self::BlockingDeleter)> {
+        self.inner
+            .blocking_delete()
+            .map(|(rp, w)| {
+                (
+                    rp,
+                    ErrorContextWrapper::new(self.info.scheme(), "".to_string(), w),
+                )
+            })
+            .map_err(|err| {
+                err.with_operation(Operation::BlockingDelete)
+                    .with_context("service", self.info.scheme())
+            })
     }
 
     fn blocking_list(&self, path: &str, args: OpList) -> Result<(RpList, Self::BlockingLister)> {
@@ -327,16 +284,12 @@ impl<A: Accessor> LayeredAccessor for ErrorContextAccessor<A> {
             .map(|(rp, os)| {
                 (
                     rp,
-                    ErrorContextWrapper {
-                        scheme: self.meta.scheme(),
-                        path: path.to_string(),
-                        inner: os,
-                    },
+                    ErrorContextWrapper::new(self.info.scheme(), path.to_string(), os),
                 )
             })
             .map_err(|err| {
                 err.with_operation(Operation::BlockingList)
-                    .with_context("service", self.meta.scheme())
+                    .with_context("service", self.info.scheme())
                     .with_context("path", path)
             })
     }
@@ -346,129 +299,210 @@ pub struct ErrorContextWrapper<T> {
     scheme: Scheme,
     path: String,
     inner: T,
+    range: BytesRange,
+    processed: u64,
+}
+
+impl<T> ErrorContextWrapper<T> {
+    fn new(scheme: Scheme, path: String, inner: T) -> Self {
+        Self {
+            scheme,
+            path,
+            inner,
+            range: BytesRange::default(),
+            processed: 0,
+        }
+    }
+
+    fn with_range(mut self, range: BytesRange) -> Self {
+        self.range = range;
+        self
+    }
 }
 
 impl<T: oio::Read> oio::Read for ErrorContextWrapper<T> {
-    fn poll_read(&mut self, cx: &mut Context<'_>, buf: &mut [u8]) -> Poll<Result<usize>> {
-        self.inner.poll_read(cx, buf).map_err(|err| {
-            err.with_operation(ReadOperation::Read)
-                .with_context("service", self.scheme)
-                .with_context("path", &self.path)
-                .with_context("read_buf", buf.len().to_string())
-        })
-    }
-
-    fn poll_seek(&mut self, cx: &mut Context<'_>, pos: SeekFrom) -> Poll<Result<u64>> {
-        self.inner.poll_seek(cx, pos).map_err(|err| {
-            err.with_operation(ReadOperation::Seek)
-                .with_context("service", self.scheme)
-                .with_context("path", &self.path)
-        })
-    }
-
-    fn poll_next(&mut self, cx: &mut Context<'_>) -> Poll<Option<Result<Bytes>>> {
-        self.inner.poll_next(cx).map_err(|err| {
-            err.with_operation(ReadOperation::Next)
-                .with_context("service", self.scheme)
-                .with_context("path", &self.path)
-        })
+    async fn read(&mut self) -> Result<Buffer> {
+        self.inner
+            .read()
+            .await
+            .map(|bs| {
+                self.processed += bs.len() as u64;
+                bs
+            })
+            .map_err(|err| {
+                err.with_operation(Operation::ReaderRead)
+                    .with_context("service", self.scheme)
+                    .with_context("path", &self.path)
+                    .with_context("range", self.range.to_string())
+                    .with_context("read", self.processed.to_string())
+            })
     }
 }
 
 impl<T: oio::BlockingRead> oio::BlockingRead for ErrorContextWrapper<T> {
-    fn read(&mut self, buf: &mut [u8]) -> Result<usize> {
-        self.inner.read(buf).map_err(|err| {
-            err.with_operation(ReadOperation::BlockingRead)
-                .with_context("service", self.scheme)
-                .with_context("path", &self.path)
-                .with_context("read_buf", buf.len().to_string())
-        })
-    }
-
-    fn seek(&mut self, pos: SeekFrom) -> Result<u64> {
-        self.inner.seek(pos).map_err(|err| {
-            err.with_operation(ReadOperation::BlockingSeek)
-                .with_context("service", self.scheme)
-                .with_context("path", &self.path)
-                .with_context("seek", format!("{pos:?}"))
-        })
-    }
-
-    fn next(&mut self) -> Option<Result<Bytes>> {
-        self.inner.next().map(|v| {
-            v.map_err(|err| {
-                err.with_operation(ReadOperation::BlockingNext)
+    fn read(&mut self) -> Result<Buffer> {
+        self.inner
+            .read()
+            .map(|bs| {
+                self.processed += bs.len() as u64;
+                bs
+            })
+            .map_err(|err| {
+                err.with_operation(Operation::BlockingReaderRead)
                     .with_context("service", self.scheme)
                     .with_context("path", &self.path)
+                    .with_context("range", self.range.to_string())
+                    .with_context("read", self.processed.to_string())
             })
-        })
     }
 }
 
-#[async_trait::async_trait]
 impl<T: oio::Write> oio::Write for ErrorContextWrapper<T> {
-    fn poll_write(&mut self, cx: &mut Context<'_>, bs: &dyn oio::WriteBuf) -> Poll<Result<usize>> {
-        self.inner.poll_write(cx, bs).map_err(|err| {
-            err.with_operation(WriteOperation::Write)
+    async fn write(&mut self, bs: Buffer) -> Result<()> {
+        let size = bs.len();
+        self.inner
+            .write(bs)
+            .await
+            .map(|_| {
+                self.processed += size as u64;
+            })
+            .map_err(|err| {
+                err.with_operation(Operation::WriterWrite)
+                    .with_context("service", self.scheme)
+                    .with_context("path", &self.path)
+                    .with_context("size", size.to_string())
+                    .with_context("written", self.processed.to_string())
+            })
+    }
+
+    async fn close(&mut self) -> Result<Metadata> {
+        self.inner.close().await.map_err(|err| {
+            err.with_operation(Operation::WriterClose)
                 .with_context("service", self.scheme)
                 .with_context("path", &self.path)
-                .with_context("write_buf", bs.remaining().to_string())
+                .with_context("written", self.processed.to_string())
         })
     }
 
-    fn poll_close(&mut self, cx: &mut Context<'_>) -> Poll<Result<()>> {
-        self.inner.poll_close(cx).map_err(|err| {
-            err.with_operation(WriteOperation::Close)
-                .with_context("service", self.scheme)
-                .with_context("path", &self.path)
-        })
-    }
-
-    fn poll_abort(&mut self, cx: &mut Context<'_>) -> Poll<Result<()>> {
-        self.inner.poll_abort(cx).map_err(|err| {
-            err.with_operation(WriteOperation::Abort)
+    async fn abort(&mut self) -> Result<()> {
+        self.inner.abort().await.map_err(|err| {
+            err.with_operation(Operation::WriterAbort)
                 .with_context("service", self.scheme)
                 .with_context("path", &self.path)
+                .with_context("processed", self.processed.to_string())
         })
     }
 }
 
 impl<T: oio::BlockingWrite> oio::BlockingWrite for ErrorContextWrapper<T> {
-    fn write(&mut self, bs: &dyn oio::WriteBuf) -> Result<usize> {
-        self.inner.write(bs).map_err(|err| {
-            err.with_operation(WriteOperation::BlockingWrite)
-                .with_context("service", self.scheme)
-                .with_context("path", &self.path)
-                .with_context("write_buf", bs.remaining().to_string())
-        })
+    fn write(&mut self, bs: Buffer) -> Result<()> {
+        let size = bs.len();
+        self.inner
+            .write(bs)
+            .map(|_| {
+                self.processed += size as u64;
+            })
+            .map_err(|err| {
+                err.with_operation(Operation::BlockingWriterWrite)
+                    .with_context("service", self.scheme)
+                    .with_context("path", &self.path)
+                    .with_context("size", size.to_string())
+                    .with_context("written", self.processed.to_string())
+            })
     }
 
-    fn close(&mut self) -> Result<()> {
+    fn close(&mut self) -> Result<Metadata> {
         self.inner.close().map_err(|err| {
-            err.with_operation(WriteOperation::BlockingClose)
+            err.with_operation(Operation::BlockingWriterClose)
                 .with_context("service", self.scheme)
                 .with_context("path", &self.path)
+                .with_context("written", self.processed.to_string())
         })
     }
 }
 
-#[async_trait::async_trait]
 impl<T: oio::List> oio::List for ErrorContextWrapper<T> {
-    fn poll_next(&mut self, cx: &mut Context<'_>) -> Poll<Result<Option<oio::Entry>>> {
-        self.inner.poll_next(cx).map_err(|err| {
-            err.with_operation(ListOperation::Next)
-                .with_context("service", self.scheme)
-                .with_context("path", &self.path)
-        })
+    async fn next(&mut self) -> Result<Option<oio::Entry>> {
+        self.inner
+            .next()
+            .await
+            .map(|bs| {
+                self.processed += bs.is_some() as u64;
+                bs
+            })
+            .map_err(|err| {
+                err.with_operation(Operation::ListerNext)
+                    .with_context("service", self.scheme)
+                    .with_context("path", &self.path)
+                    .with_context("listed", self.processed.to_string())
+            })
     }
 }
 
 impl<T: oio::BlockingList> oio::BlockingList for ErrorContextWrapper<T> {
     fn next(&mut self) -> Result<Option<oio::Entry>> {
-        self.inner.next().map_err(|err| {
-            err.with_operation(ListOperation::BlockingNext)
+        self.inner
+            .next()
+            .map(|bs| {
+                self.processed += bs.is_some() as u64;
+                bs
+            })
+            .map_err(|err| {
+                err.with_operation(Operation::BlockingListerNext)
+                    .with_context("service", self.scheme)
+                    .with_context("path", &self.path)
+                    .with_context("listed", self.processed.to_string())
+            })
+    }
+}
+
+impl<T: oio::Delete> oio::Delete for ErrorContextWrapper<T> {
+    fn delete(&mut self, path: &str, args: OpDelete) -> Result<()> {
+        self.inner.delete(path, args).map_err(|err| {
+            err.with_operation(Operation::DeleterDelete)
                 .with_context("service", self.scheme)
-                .with_context("path", &self.path)
+                .with_context("path", path)
+                .with_context("deleted", self.processed.to_string())
         })
+    }
+
+    async fn flush(&mut self) -> Result<usize> {
+        self.inner
+            .flush()
+            .await
+            .map(|n| {
+                self.processed += n as u64;
+                n
+            })
+            .map_err(|err| {
+                err.with_operation(Operation::DeleterFlush)
+                    .with_context("service", self.scheme)
+                    .with_context("deleted", self.processed.to_string())
+            })
+    }
+}
+
+impl<T: oio::BlockingDelete> oio::BlockingDelete for ErrorContextWrapper<T> {
+    fn delete(&mut self, path: &str, args: OpDelete) -> Result<()> {
+        self.inner.delete(path, args).map_err(|err| {
+            err.with_operation(Operation::DeleterDelete)
+                .with_context("service", self.scheme)
+                .with_context("path", path)
+                .with_context("deleted", self.processed.to_string())
+        })
+    }
+
+    fn flush(&mut self) -> Result<usize> {
+        self.inner
+            .flush()
+            .map(|n| {
+                self.processed += n as u64;
+                n
+            })
+            .map_err(|err| {
+                err.with_operation(Operation::DeleterFlush)
+                    .with_context("service", self.scheme)
+                    .with_context("deleted", self.processed.to_string())
+            })
     }
 }
