@@ -19,29 +19,25 @@ mod api;
 pub use api::BlockingWrite;
 pub use api::BlockingWriter;
 pub use api::Write;
-pub use api::WriteExt;
-pub use api::WriteOperation;
 pub use api::Writer;
 
-mod compose_write;
-pub use compose_write::ThreeWaysWriter;
+mod multipart_write;
+pub use multipart_write::MultipartPart;
+pub use multipart_write::MultipartWrite;
+pub use multipart_write::MultipartWriter;
 
-mod multipart_upload_write;
-pub use multipart_upload_write::MultipartUploadPart;
-pub use multipart_upload_write::MultipartUploadWrite;
-pub use multipart_upload_write::MultipartUploadWriter;
-
-mod append_object_write;
-pub use append_object_write::AppendObjectWrite;
-pub use append_object_write::AppendObjectWriter;
+mod append_write;
+pub use append_write::AppendWrite;
+pub use append_write::AppendWriter;
 
 mod one_shot_write;
 pub use one_shot_write::OneShotWrite;
 pub use one_shot_write::OneShotWriter;
 
-mod exact_buf_write;
-pub use exact_buf_write::ExactBufWriter;
+mod block_write;
+pub use block_write::BlockWrite;
+pub use block_write::BlockWriter;
 
-mod range_write;
-pub use range_write::RangeWrite;
-pub use range_write::RangeWriter;
+mod position_write;
+pub use position_write::PositionWrite;
+pub use position_write::PositionWriter;
