@@ -231,6 +231,15 @@ impl Access for AzdlsBackend {
             .set_name(&self.core.filesystem)
             .set_native_capability(Capability {
                 stat: true,
+                stat_has_cache_control: true,
+                stat_has_content_length: true,
+                stat_has_content_type: true,
+                stat_has_content_encoding: true,
+                stat_has_content_range: true,
+                stat_has_etag: true,
+                stat_has_content_md5: true,
+                stat_has_last_modified: true,
+                stat_has_content_disposition: true,
 
                 read: true,
 
@@ -244,6 +253,9 @@ impl Access for AzdlsBackend {
                 rename: true,
 
                 list: true,
+                list_has_etag: true,
+                list_has_content_length: true,
+                list_has_last_modified: true,
 
                 shared: true,
 
