@@ -51,6 +51,8 @@ impl Access for DropboxBackend {
             .set_root(&self.core.root)
             .set_native_capability(Capability {
                 stat: true,
+                stat_has_last_modified: true,
+                stat_has_content_length: true,
 
                 read: true,
 
@@ -62,6 +64,8 @@ impl Access for DropboxBackend {
 
                 list: true,
                 list_with_recursive: true,
+                list_has_last_modified: true,
+                list_has_content_length: true,
 
                 copy: true,
 
