@@ -15,11 +15,21 @@
 // specific language governing permissions and limitations
 // under the License.
 
-mod backend;
-pub use backend::FtpBuilder as Ftp;
-pub use backend::FtpConfig;
-
+#[cfg(feature = "services-ftp")]
+mod delete;
+#[cfg(feature = "services-ftp")]
 mod err;
+#[cfg(feature = "services-ftp")]
 mod lister;
-mod util;
+#[cfg(feature = "services-ftp")]
+mod reader;
+#[cfg(feature = "services-ftp")]
 mod writer;
+
+#[cfg(feature = "services-ftp")]
+mod backend;
+#[cfg(feature = "services-ftp")]
+pub use backend::FtpBuilder as Ftp;
+
+mod config;
+pub use config::FtpConfig;
