@@ -24,11 +24,14 @@ use std::time::Duration;
 use backon::ExponentialBuilder;
 use backon::Retryable;
 use bytes::Bytes;
+use http::header::CONTENT_ENCODING;
 use http::header::CONTENT_LENGTH;
 use http::header::CONTENT_TYPE;
 use http::header::HOST;
 use http::header::IF_MATCH;
+use http::header::IF_MODIFIED_SINCE;
 use http::header::IF_NONE_MATCH;
+use http::header::IF_UNMODIFIED_SINCE;
 use http::Request;
 use http::Response;
 use once_cell::sync::Lazy;
@@ -221,6 +224,20 @@ impl GcsCore {
             req = req.header(IF_NONE_MATCH, if_none_match);
         }
 
+        if let Some(if_modified_since) = args.if_modified_since() {
+            req = req.header(
+                IF_MODIFIED_SINCE,
+                format_datetime_into_http_date(if_modified_since),
+            );
+        }
+
+        if let Some(if_unmodified_since) = args.if_unmodified_since() {
+            req = req.header(
+                IF_UNMODIFIED_SINCE,
+                format_datetime_into_http_date(if_unmodified_since),
+            );
+        }
+
         let req = req.body(Buffer::new()).map_err(new_request_build_error)?;
 
         Ok(req)
@@ -251,6 +268,7 @@ impl GcsCore {
             storage_class: self.default_storage_class.as_deref(),
             cache_control: op.cache_control(),
             content_type: op.content_type(),
+            content_encoding: op.content_encoding(),
             metadata: op.user_metadata(),
         };
 
@@ -331,6 +349,10 @@ impl GcsCore {
             req = req.header(CONTENT_TYPE, content_type);
         }
 
+        if let Some(content_encoding) = args.content_encoding() {
+            req = req.header(CONTENT_ENCODING, content_encoding);
+        }
+
         if let Some(acl) = &self.predefined_acl {
             req = req.header(X_GOOG_ACL, acl);
         }
@@ -623,6 +645,8 @@ pub struct InsertRequestMetadata<'a> {
     #[serde(skip_serializing_if = "Option::is_none")]
     content_type: Option<&'a str>,
     #[serde(skip_serializing_if = "Option::is_none")]
+    content_encoding: Option<&'a str>,
+    #[serde(skip_serializing_if = "Option::is_none")]
     storage_class: Option<&'a str>,
     #[serde(skip_serializing_if = "Option::is_none")]
     cache_control: Option<&'a str>,
@@ -633,6 +657,7 @@ pub struct InsertRequestMetadata<'a> {
 impl InsertRequestMetadata<'_> {
     pub fn is_empty(&self) -> bool {
         self.content_type.is_none()
+            && self.content_encoding.is_none()
             && self.storage_class.is_none()
             && self.cache_control.is_none()
             && self.metadata.is_none()
