@@ -205,11 +205,15 @@ impl Access for HuggingfaceBackend {
         am.set_scheme(Scheme::Huggingface)
             .set_native_capability(Capability {
                 stat: true,
+                stat_has_content_length: true,
+                stat_has_last_modified: true,
 
                 read: true,
 
                 list: true,
                 list_with_recursive: true,
+                list_has_content_length: true,
+                list_has_last_modified: true,
 
                 shared: true,
 
