@@ -261,6 +261,16 @@ impl Access for ObsBackend {
                 stat: true,
                 stat_with_if_match: true,
                 stat_with_if_none_match: true,
+                stat_has_cache_control: true,
+                stat_has_content_length: true,
+                stat_has_content_type: true,
+                stat_has_content_encoding: true,
+                stat_has_content_range: true,
+                stat_has_etag: true,
+                stat_has_content_md5: true,
+                stat_has_last_modified: true,
+                stat_has_content_disposition: true,
+                stat_has_user_metadata: true,
 
                 read: true,
 
@@ -292,6 +302,7 @@ impl Access for ObsBackend {
 
                 list: true,
                 list_with_recursive: true,
+                list_has_content_length: true,
 
                 presign: true,
                 presign_stat: true,
