@@ -15,6 +15,11 @@
 // specific language governing permissions and limitations
 // under the License.
 
+#[cfg(feature = "services-redis")]
 mod backend;
+#[cfg(feature = "services-redis")]
 pub use backend::RedisBuilder as Redis;
-pub use backend::RedisConfig;
+mod config;
+#[cfg(feature = "services-redis")]
+mod core;
+pub use config::RedisConfig;
