@@ -204,6 +204,11 @@ impl Access for UpyunBackend {
             .set_root(&self.core.root)
             .set_native_capability(Capability {
                 stat: true,
+                stat_has_content_length: true,
+                stat_has_content_type: true,
+                stat_has_content_md5: true,
+                stat_has_cache_control: true,
+                stat_has_content_disposition: true,
 
                 create_dir: true,
 
@@ -225,6 +230,9 @@ impl Access for UpyunBackend {
 
                 list: true,
                 list_with_limit: true,
+                list_has_content_length: true,
+                list_has_content_type: true,
+                list_has_last_modified: true,
 
                 shared: true,
 
